#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "cubature.h"

#ifndef SPEED_OF_LIGHT
#define SPEED_OF_LIGHT 2.99792E8
#endif

struct fn_data{
	double w;
	double theta_0;
	double phi_0;
	double sigma;
	double corr_length;
	double corr_length_y; //For the aniso case
	int order;

};

double alpha_0(double x, double w);
int mrc_gauss(double *result, double w, double theta_0, double *k, double *q, double sigma, double corr_length, int order);
int mrc_exp(double *result, double w, double theta_0, double *k, double *q, double sigma, double corr_length, int order);
int f_exp(unsigned ndim, const double *x, void *fdata, unsigned fdim, double *fval);
int f_gauss(unsigned ndim, const double *x, void *fdata, unsigned fdim, double *fval);
int mrc_gauss_aniso(double *result, double w, double theta_0, double *k, double *q, double sigma, double corr_length_x, double corr_length_y, int order);
int integral(double *ret_val, double theta_0, double phi_0, double w, double sigma, double corr_length, double corr_length_y,int order, int corr_fn_choice, double abs_err, double rel_err);
int f_gauss_aniso(unsigned ndim, const double *x, void *fdata, unsigned fdim, double *fval);
int f_gauss_new(unsigned ndim, const double *x, void *fdata, unsigned fdim, double *fval);

int main(int argc, char *argv[]){
	double c = SPEED_OF_LIGHT;
	double sigma;
	double w;
	double theta_0;
	double phi_0;
	double corr_length;
	int order;
	int num_theta;
	int num_phi;
	int corr_fn_choice;
	double abs_err;
	double rel_err;
	double corr_length_y;
	if(argc < 10){
		fprintf(stderr,"Insufficient number of arguments\n");
		return 1;
	}

	//Get arguments from command line
	if(!sscanf(argv[1],"%lf",&w))
		return 2;
	if(!sscanf(argv[2],"%lf",&sigma))
		return 3;
	if(!sscanf(argv[3],"%lf",&theta_0))
		return 4;
	if(!sscanf(argv[4],"%lf",&phi_0))
		return 5;
	if(!sscanf(argv[5],"%lf", &corr_length))
		return 6;
	if(!sscanf(argv[6],"%d", &order))
		return 7;
	if(!sscanf(argv[7],"%d", &corr_fn_choice))
		return 8;
	if(!sscanf(argv[8],"%lf", &abs_err))
		return 9;
	if(!sscanf(argv[9],"%lf", &rel_err))
		return 10;
	if(!sscanf(argv[10],"%lf",&corr_length_y))
		return 11;

	//Calculate specular contribution to dR/dO
	double spec_power = exp(-4*pow( (w/c)*sigma*cos(theta_0),2));
	
	double ret_val;
	int errc;
	//Calculate incoherent contribution
	errc = integral(&ret_val, theta_0, phi_0, w, sigma, corr_length, corr_length_y, order, corr_fn_choice, abs_err,rel_err);

	fprintf(stdout, "%e ", ret_val+spec_power);


}


int integral(double *ret_val, double theta_0, double phi_0, double w, double sigma, double corr_length, double corr_length_y, int order, int corr_fn_choice, double abs_err, double rel_err){

	int (*integr_fn)(unsigned, const double *, void *,
      unsigned, double*);

	unsigned max_eval = 100000000;
	double c= SPEED_OF_LIGHT;
	double result;
	double res_error1, res_error2;
	struct fn_data input_data;
	input_data.w=w;
	input_data.theta_0 = theta_0;
	input_data.phi_0 = phi_0;
	input_data.sigma = sigma;
	input_data.corr_length = corr_length;
	input_data.corr_length_y = corr_length_y;
	input_data.order = order;
	if(corr_fn_choice == 0)
		integr_fn = &f_exp;
	else if(corr_fn_choice == 1)
		integr_fn = &f_gauss;
	else if(corr_fn_choice == 2)
		integr_fn = &f_gauss_aniso;
	else
		return 1;

	double klen = (w/c)*sin(theta_0);


	/*
	Integrate over 20.0/corr_length or over the entire domain, whichever is smallest. If the first, the rest of the domain will be
	integrated over in a second run. This because, for high correlation lengths, the numerical integrator will miss the sharp peak if
	integrating over the entire domain.*/

	fprintf(stderr, "error: ");
	double xmin[2] = {0.0,0.0};
	double xmax[2] = {(w/c),2*M_PI};
	int errc = hcubature(1, integr_fn, &input_data, 2, xmin, xmax, max_eval, abs_err, rel_err, ERROR_LINF, &result, &res_error1);
	fprintf(stderr, "%e ",res_error1);			
	*ret_val = result;

	fprintf(stderr,"\n");
	
	return 0;

}

int mrc_gauss(double *result, double w, double theta_0, double *k, double *q, double sigma, double corr_length, int order){
	
	double c = SPEED_OF_LIGHT;
	double q_len = sqrt(q[0]*q[0]+q[1]*q[1]);
	double k_len = sqrt(k[0]*k[0]+k[1]*k[1]);
	double q_min_k = sqrt( (q[0]-k[0])*(q[0]-k[0])+(q[1]-k[1])*(q[1]-k[1]));
	
	double aq = alpha_0(q_len,w);
	double ak = alpha_0(k_len,w);

	double a = pow((aq+ak)*sigma,2);
	double b = pow( q_min_k*corr_length/2,2);
	double prefac = exp(-a);
	prefac *= corr_length*corr_length/(4*M_PI*cos(theta_0));
	prefac /= pow(ak+aq,2);
	prefac *= pow( (w/c)*(w/c) +ak*aq-q[0]*k[0]-q[1]*k[1], 2);

	int i;
	*result = 0.0;
	double fac = 1.0;
	for(i=1;i<=order;i++){
		fac *= (a/i);
		*result += (fac/i)*exp(-b/i);
	}
	*result *= prefac;
	return 0;
}

int mrc_gauss_aniso(double *result, double w, double theta_0, double *k, double *q, double sigma, double corr_length_x, double corr_length_y, int order){
	
	double c = SPEED_OF_LIGHT;
	double q_len = sqrt(q[0]*q[0]+q[1]*q[1]);
	double k_len = sqrt(k[0]*k[0]+k[1]*k[1]);
	double q_min_k = sqrt( (q[0]-k[0])*(q[0]-k[0])+(q[1]-k[1])*(q[1]-k[1]));
	
	double aq = alpha_0(q_len,w);
	double ak = alpha_0(k_len,w);

	double a = pow((aq+ak)*sigma,2);
	double b = 0.25*(pow(corr_length_x*(q[0]-k[0]) ,2)+pow(corr_length_y*(q[1]-k[1]) ,2));
	double prefac = exp(-a);
	prefac *= corr_length_x*corr_length_y/(4*M_PI*cos(theta_0));
	prefac /= pow(ak+aq,2);
	prefac *= pow( (w/c)*(w/c) +ak*aq-q[0]*k[0]-q[1]*k[1], 2);

	int i;
	*result = 0.0;
	double fac = 1.0;	
	for(i=1;i<=order;i++){
		fac *= (a/i);		
		*result += (fac/i)*exp(-b/i);
	}
		
	*result *= prefac;

	
	return 0;
}
int f_exp(unsigned ndim, const double *x, void *fdata, unsigned fdim, double *fval){
	struct fn_data *fn_input_data = (struct fn_data*)fdata;
	double c = SPEED_OF_LIGHT;
	double w = fn_input_data->w;
	double theta_0 = fn_input_data->theta_0;
	double phi_0 = fn_input_data->phi_0;
	double sigma = fn_input_data->sigma;
	double corr_length = fn_input_data->corr_length;
	int order = fn_input_data->order;

	double k[2] = {(w/c)*sin(theta_0)*cos(phi_0), (w/c)*sin(theta_0)*sin(phi_0)};
	double q[2] = {x[0]*cos(x[1]), x[0]*sin(x[1])};
	double q_len = sqrt(q[0]*q[0]+q[1]*q[1]);
	double alph = alpha_0(q_len,w);
	int errc = 0;
	//Check if we are outside the integration domain
	if(alph > 0.0){
		errc = mrc_exp(fval, w, theta_0,k,q,sigma,corr_length,order);
		*fval *=x[0];		
		*fval /= ((w/c)*alpha_0(q_len,w));
	}
	else
		*fval = 0.0;
	return errc;
}

int f_gauss(unsigned ndim, const double *x, void *fdata, unsigned fdim, double *fval){
	struct fn_data *fn_input_data = (struct fn_data*)fdata;
	double c = SPEED_OF_LIGHT;
	double w = fn_input_data->w;
	double theta_0 = fn_input_data->theta_0;
	double phi_0 = fn_input_data->phi_0;
	double sigma = fn_input_data->sigma;
	double corr_length = fn_input_data->corr_length;
	int order = fn_input_data->order;

	double k[2] = {(w/c)*sin(theta_0)*cos(phi_0), (w/c)*sin(theta_0)*sin(phi_0)};
	double q[2] = {x[0]*cos(x[1]), x[0]*sin(x[1])};
	double q_len = sqrt(q[0]*q[0]+q[1]*q[1]);
	double alph = alpha_0(q_len,w);
	int errc = 0;	
	if(alph > 0.0){
		errc = mrc_gauss(fval, w, theta_0,k,q,sigma,corr_length,order);
		*fval *= x[0];
		*fval /= ((w/c)*alpha_0(q_len,w));
	}
	else
		*fval = 0.0;
	return errc;
}
int f_gauss_aniso(unsigned ndim, const double *x, void *fdata, unsigned fdim, double *fval){
	struct fn_data *fn_input_data = (struct fn_data*)fdata;
	double c = SPEED_OF_LIGHT;
	double w = fn_input_data->w;
	double theta_0 = fn_input_data->theta_0;
	double phi_0 = fn_input_data->phi_0;
	double sigma = fn_input_data->sigma;
	double corr_length = fn_input_data->corr_length;
	double corr_length_y = fn_input_data->corr_length_y;
	int order = fn_input_data->order;
	
	


	double k[2] = {(w/c)*sin(theta_0)*cos(phi_0), (w/c)*sin(theta_0)*sin(phi_0)};
	double q[2] = {x[0]*cos(x[1]), x[0]*sin(x[1])};
	double q_len = sqrt(q[0]*q[0]+q[1]*q[1]);
	double k_len = sqrt(k[0]*k[0]+k[1]*k[1]);
	double alph = alpha_0(q_len,w);

	int errc = 0;	
	if(alph <= 0.0)
		*fval=0.0;
	else{
		errc = mrc_gauss_aniso(fval, w, theta_0,k,q,sigma,corr_length,corr_length_y,order);
		*fval *= x[0];
		*fval /= ((w/c)*alpha_0(q_len,w));
	}
	return errc;
}
int mrc_exp(double *result, double w, double theta_0, double *k, double *q, double sigma, double corr_length, int order){
	
	double c = SPEED_OF_LIGHT;
	double q_len = sqrt(q[0]*q[0]+q[1]*q[1]);
	double k_len = sqrt(k[0]*k[0]+k[1]*k[1]);
	double q_min_kr = sqrt( (q[0]-k[0])*(q[0]-k[0])+(q[1]-k[1])*(q[1]-k[1]))/(2*M_PI);
	
	double aq = alpha_0(q_len,w);
	double ak = alpha_0(k_len,w);

	double a = pow((aq+ak)*sigma,2);
	double b = 2*M_PI*corr_length;	
	double prefac = exp(-a)/(4*M_PI*M_PI*cos(theta_0));
	prefac *= pow( (w/c)*(w/c) + aq*ak-q[0]*k[0]-q[1]*k[1],2);
	prefac /= pow(ak+aq,2);
	prefac *= 1.0/(2.0*M_PI);

	int i;
	double alpha;	
	*result=0.0;
	double fac = 1.0;
	for(i=1;i<=order;i++){
		alpha = ((double)i)/b;
		fac *= (a/i);
		*result += fac*alpha/pow( alpha*alpha + q_min_kr*q_min_kr,3.0/2);
	}
	*result *= prefac;
	return 0;
}

/** \brief alpha_0 function.

*/
double alpha_0(double x, double w){
  double c = SPEED_OF_LIGHT;
  double tmp = (w/c)*(w/c)-x*x;
	if(tmp < 0)
		tmp = 0.0; 
	return sqrt(tmp);

}

