import math
import numpy as np
import scipy.integrate as integ
import scipy.interpolate as itp
from mdrc import mdrc_kirch #Import the function calculating mdrc on MxN grid

#Integrand when integrating over A_G    
def integrand_g(Th,p,args):
    theta_0=args[0]
    phi_0=args[1]
    p1=p*math.cos(Th)
    p2=p*math.sin(Th)
    plen_sq = p*p
    alph=math.sqrt(1.0-plen_sq)
    qq1 = p1*math.cos(theta_0)+alph*math.sin(theta_0)
    qq2 = p2
    q1=qq1*math.cos(phi_0)-qq2*math.sin(phi_0)
    q2=qq1*math.sin(phi_0)+qq2*math.cos(phi_0)
    return p*f_mdrc(q1,q2)/alph
    
#Set parameters
M=600
N=500
theta_0=20.0*math.pi/180
phi_0 = 0.0*math.pi/180
Theta_lim = 2.5*math.pi/180
delta_r = 0.1
a_r = 5.0
order = 500
corr_func = 'gauss'
a_r_y = 2.0

#Arguments to pass to the integrand when integrating over A_G
argsi = []
argsi.append(theta_0)
argsi.append(phi_0)

#Get incoherent part of MDRC on MxN grid.
qx,qy,mdrc = mdrc_kirch(M,N,theta_0,a_r,delta_r,corr_func,order)

#Interpolate to get a function mdrc(qx,qy)
f_mdrc = itp.interp2d(qx,qy,mdrc,kind='cubic')

#Integrand when calculating U.
integrand_u_cart = lambda qx,qy: f_mdrc(qx,qy)/math.sqrt(1-qx*qx-qy*qy)     

#Calculate integral over incoherent part of MDRC over all scattering angles. Cartesian.
a = integ.dblquad(integrand_u_cart, -1,1, lambda x: -math.sqrt(1-x*x), lambda x: math.sqrt(1-x*x),args=(), epsabs=1.49e-03, epsrel=1.49e-03)    

#Calculate integral over incoherent part of MDRC over A_G. Polar coordinates.
b = integ.dblquad(integrand_g, 0,math.sin(Theta_lim), lambda x: 0, lambda x: 2*math.pi,args=(argsi,), epsabs=1.49e-08, epsrel=1.49e-08)

#Add coherent contributions.
u = a[0]+np.exp(-np.power(4*math.pi*delta_r*math.cos(theta_0),2)) 
g = b[0]+np.exp(-np.power(4*math.pi*delta_r*math.cos(theta_0),2)) 

#Print U
print u
#Print haze
print (1-g/u)