import math
import numpy as np
import matplotlib.pyplot as plt

#Calculates haze by approximate expressions.
#
#Inputs:
#
#delt_r : rms-roughness divided by wavelength: delta/lambda
#a_r : corr. length divided by wavelength: a/lambda
#theta_0 : angle of incidence.
#Theta_lim : Limit angle. See 'definition of haze'.
#corr_func : 'exp' for exponential and 'gauss' for gaussian correlation func.
#num_terms : Number of terms in the sum.
#
#Returns: haze as number, array or grid, depending on inputs.
def haze_approx(delt_r,a_r,theta_0,Theta_lim,corr_func,num_terms):

    #Some factors that are useful
    alph_k = 2*math.pi*np.cos(theta_0)    
    rf= 4*alph_k*alph_k*delt_r*delt_r
    
    #Radius of integration area in q_1q_2
    delta_q = 2*math.pi*np.sin(Theta_lim)*np.sqrt(np.cos(theta_0))
    df = a_r*a_r*delta_q*delta_q
    
    #Apply sum formula.
    tmp = 0
    fac = 1.0
    if corr_func == 'gauss':
        for i in range(1,approx_order):
            fac *= rf/i  
            tmp += fac*(1-np.exp(-df/(4*i)))
            
    elif corr_func == 'exp':
       for i in range(1,approx_order):
            fac *= rf/i 
            tmp += fac*(1-np.power(1+df/(i*i),-0.5))
    else:
        return 'no correlation function given'
    
    #Haze is 1 minus (coherent part + incoherent part) of gloss
    h_approx = 1.0-np.exp(-rf)*(1+tmp)
    
    return h_approx

#Set input values
delt_red = 0.3
theta_0 = 20.0*math.pi/180
Theta_lim = 2.5*math.pi/180
approx_order = 1000
corr_func = 'gauss'
cl_red=np.linspace(0.1,100,200) #Corr. length is an array in this example.

#Approximate haze
haze = haze_approx(delt_red,cl_red,theta_0,Theta_lim,corr_func,approx_order)

#Plot
plt.plot(cl_red,haze)
plt.xscale('log')
plt.yscale('log')
plt.ylim((1E-3,1))