
import math
import matplotlib.pyplot as plt
import numpy as np
import scipy.special as sps
import scipy.integrate as integ

#Gaussian anisotropic corr. function
def W_gauss_aniso(ux,uy,args):
    ax=args[0]
    ay=args[1]
    return math.exp(-(ux/ax)*(ux/ax)-(uy/ay)*(uy/ay))

#Function that gives the upper integral limit of |v| as a function of Theta_v.
#Left for the reader to specify something more sophisticated.
def vmax(Th):
    return 15

#Evaluates the integrand for a given value of |v| and Th_v
def integrand(v,Th_v,args):
    alph2=args[0]
    d2=args[1]
    dp=args[2]
    costh0=args[3]
    s=args[4]
    c=args[5]
    W=args[6]
    W_args=args[7]
    
    vx=v*math.cos(Th_v) #Get cartesian x-value of integration variable
    vy=v*math.sin(Th_v) #Get cartesian y-value of integration variable
    ux=(vx*c/costh0 - s*vy)/dp #Get corresponding Delta x_1 value
    uy=(s*vx/costh0+c*vy)/dp    #Get corresponding Delta x_2 value
    
    y=(math.exp(4*alph2*d2*W(ux,uy,W_args))-1)*sps.j1(v) #Evaluate integrand
    return y
    


lamb = 457.9*1E-9
wc = 2*math.pi/lamb

##############################################################
#INPUT PARAMETERS

theta_0 = 30.0*math.pi/180
phi_0 = 0.0*math.pi/180
delta = 0.2*lamb
Theta_lim = 2.5*math.pi/180

#Choose correlation function
corr_func = W_gauss_aniso 

#Arguments to be passed to the correlation function.
#Here: Correlation lengths for x- and y-direction, respectively.
corr_func_args=np.array([7*lamb,14*lamb]) 

#Error tolerance for scipy.integrate.dblquad()
abs_err = 1.0E-6
rel_err = 1.0E-6
###########################################################
argsi=[]
argsi.append(math.pow(wc*math.cos(theta_0),2))
argsi.append(math.pow(delta,2))
argsi.append(wc*math.sin(Theta_lim))
argsi.append(math.cos(theta_0))
argsi.append(math.sin(phi_0))
argsi.append(math.cos(phi_0))
argsi.append(corr_func)
argsi.append(corr_func_args)


N=50
phi_arr=np.linspace(0,math.pi/2,N)
haze_arr=np.zeros(N)


for i in range(0,N):
    argsi[4]=math.sin(phi_arr[i])
    argsi[5]=math.cos(phi_arr[i])
    a = integ.dblquad(integrand,0,2*math.pi, lambda x: 0, vmax,args=(argsi,),epsabs=abs_err, epsrel=rel_err)
    haze_arr[i] = 1-math.exp(-4*argsi[0]*argsi[1])*(1+a[0]/(2*math.pi))    

plt.plot(phi_arr*180/math.pi,haze_arr)
